#include "BinaryDeSerializer.hpp"
#include "BinaryUtils.hpp"

#include <iostream>

template<class t_type>
bool DeSerializePOD( t_type& o_out, std::istream& i_istream, std::streamsize i_nMaxPos)
{
	// Check if stream is valid
	if ( !i_istream) return false;
	// Check if the current position has exceeded the specified size for this chunk
	if ( i_istream.tellg() >= i_nMaxPos) return false;

	std::streamsize nTypeSize = sizeof( t_type);
	// i_istream.read( reinterpret_cast<char*>(&nTypeSize), 4);
	i_istream.read( reinterpret_cast<char*>(&o_out), nTypeSize);
	return i_istream.good();
}

//////////////////////////////////////////////////////////////////////////////////////
// CBinaryDeSerializer : Implmentation
//////////////////////////////////////////////////////////////////////////////////////

PSF::CBinaryDeSerializer::CBinaryDeSerializer( std::istream& i_istream, bool i_bSearch) :
	m_istream( i_istream),
	m_bSearch( i_bSearch)
{
	m_istream.read( reinterpret_cast<char*>(&m_nSize), 4);
	m_nStartPos = m_istream.tellg();
}

PSF::CBinaryDeSerializer::~CBinaryDeSerializer()
{
	// seek to end when done
	m_istream.seekg( m_nStartPos + m_nSize, std::ios_base::beg);
}

bool PSF::CBinaryDeSerializer::FindElement( const std::wstring& i_wstrName)
{
	// Check if stream is valid
	if ( !m_istream) return false;

	unsigned int nNameHash = HashString( i_wstrName);

	// Check if the current position has exceeded the specified size for this chunk
	if ( m_istream.tellg() < m_nStartPos + m_nSize)
	{
		// Check if the current position matches the requested 
		unsigned int nReadHash;
		m_istream.read( reinterpret_cast<char*>(&nReadHash), 4);
		if ( nReadHash == nNameHash) return true;
	}

	// The position didn't match can we search?
	if ( !m_bSearch) return false;

	// seek to start of block when starting search
	m_istream.seekg( m_nStartPos, std::ios_base::beg);

	// Iterate through blocks
	while ( &m_istream && m_istream.tellg() < m_nStartPos + m_nSize)
	{
		unsigned int nReadHash;
		m_istream.read( reinterpret_cast<char*>(&nReadHash), 4);
		if ( nReadHash == nNameHash) return true;
	
		unsigned int nSkip;
		m_istream.read( reinterpret_cast<char*>(&nSkip), 4);
		m_istream.seekg( nSkip, std::ios_base::cur);
	}

	return false;
}


bool PSF::CBinaryDeSerializer::DeSerialize( int& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( unsigned int& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( bool& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( char& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( unsigned char& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( short& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( unsigned short& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( long& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( unsigned long& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( float& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( double& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

bool PSF::CBinaryDeSerializer::DeSerialize( wchar_t& o_out)
{
	return DeSerializePOD( o_out, m_istream, m_nStartPos + m_nSize);
}

std::auto_ptr<PSF::IDeSerializer> PSF::CBinaryDeSerializer::DeSerializeCompound( const std::wstring& i_wstrName, 
																		    const Loki::TypeInfo& i_type)
{
	if ( !FindElement( i_wstrName)) return std::auto_ptr<IDeSerializer>( 0);
	return std::auto_ptr<IDeSerializer>( new CBinaryDeSerializer( m_istream, true));
}

std::auto_ptr<PSF::IDeSerializer> PSF::CBinaryDeSerializer::DeSerializeRange( const std::wstring& i_wstrName, 
																	     const Loki::TypeInfo& i_type)
{
	if ( !FindElement( i_wstrName)) return std::auto_ptr<IDeSerializer>( 0);
	return std::auto_ptr<IDeSerializer>( new CBinaryDeSerializer( m_istream, false));
}
