#ifndef CHECKER_HPP
#define CHECKER_HPP

#include <string>
#include <memory>
#include "Loki/TypeInfo.h"

namespace PSF
{
	class IChecker
	{
	public:
		virtual ~IChecker() {};

		// Type Error reporting functions

		// TypeMissing indicates that no type info was present in the data stream
		virtual void TypeMissing() const = 0;
		// TypeMismatch indicates that the type of data in the stream does not match
		// the type supplied by the program
		virtual void TypeMismatch() const = 0;
		// TypeUnknown indicates that the program failed to supply a type identifier
		virtual void TypeUnknown() const = 0;
		// SemanticError indicates that the format of data in the stream does not
		// match the expected type, fx. the stream contains the string 'abc', but
		// the program expects an integer
		virtual void SematicError() const = 0;

		// Range Error reporting functions

		// RangeSizeError indicates that an error occured while deserializing a range.
		// The expected size of the range did not match the number of items deserialized
		// from that range
		virtual void RangeSizeError() const  = 0;
	};

	class CCheckerNull : public IChecker
	{
	public:
		virtual ~CCheckerNull();

		// Type Error reporting functions
		virtual void TypeMissing() const;
		virtual void TypeMismatch() const;
		virtual void TypeUnknown() const;
		virtual void SematicError() const;

		// Range Error reporting functions
		virtual void RangeSizeError() const;
	};

	class CCheckerAssert : public IChecker
	{
	public:
		virtual ~CCheckerAssert();

		// Type Error reporting functions
		virtual void TypeMissing() const;
		virtual void TypeMismatch() const;
		virtual void TypeUnknown() const;
		virtual void SematicError() const;

		// Range Error reporting functions
		virtual void RangeSizeError() const;
	};

	class CCheckerException : public IChecker
	{
	public:
		virtual ~CCheckerException();

		// Type Error reporting functions
		virtual void TypeMissing() const;
		virtual void TypeMismatch() const;
		virtual void TypeUnknown() const;
		virtual void SematicError() const;

		// Range Error reporting functions
		virtual void RangeSizeError() const;
	};
};
#endif