#ifndef COMPOUND_HPP
#define COMPOUND_HPP

#include "Loki/Typelist.h"
#include "Loki/HierarchyGenerators.h"

namespace PSF
{
	template<class t_type>
	class TValue
	{
	public:
		t_type m_value;
	};

	template<class t_TypeList,class t_Unique,class t_Base = Loki::EmptyType>
	class TCompound : public Loki::GenScatterHierarchy< t_TypeList, TValue>, public t_Base
	{
	public:
		enum { number_of_members = Loki::TL::Length<t_TypeList>::value};
		static const wchar_t* ms_namelist[];
		typedef t_TypeList TypeList;
		typedef t_Unique Unique;
		typedef t_Base Base;
		typedef Loki::GenScatterHierarchy< t_TypeList, TValue> ScatterBase;
		
		template<unsigned int t_i>
		Loki::TL::TypeAt<TypeList,t_i>::Result& Member()
		{
			return Loki::Field<t_i>(static_cast<ScatterBase&>(*this)).m_value;
		}

		template<unsigned int t_i>
		const Loki::TL::TypeAt<TypeList,t_i>::Result& Member() const
		{
			return Loki::Field<t_i>(static_cast<const ScatterBase&>(*this)).m_value;
		}
	};
};

#define ACCESSORS_1(T1)\
const Loki::TL::TypeAt<TypeList,0>::Result& T1() const { return Member<0>();}\
Loki::TL::TypeAt<TypeList,0>::Result& T1() { return Member<0>();}\
const Loki::TL::TypeAt<TypeList,0>::Result& Get##T1() const { return Member<0>();}\
void Set##T1( const Loki::TL::TypeAt<TypeList,0>::Result& i_in) { Member<0>() = i_in;}

#define ACCESSORS_2(T1,T2) ACCESSORS_1(T1)\
const Loki::TL::TypeAt<TypeList,1>::Result& T2() const { return Member<1>();}\
Loki::TL::TypeAt<TypeList,1>::Result& T2() { return Member<1>();}\
const Loki::TL::TypeAt<TypeList,1>::Result& Get##T2() const { return Member<1>();}\
void Set##T2( const Loki::TL::TypeAt<TypeList,1>::Result& i_in) { Member<1>() = i_in;}

#define ACCESSORS_3(T1,T2,T3) ACCESSORS_2(T1,T2)\
const Loki::TL::TypeAt<TypeList,2>::Result& T3() const { return Member<2>();}\
Loki::TL::TypeAt<TypeList,2>::Result& T3() { return Member<2>();}\
const Loki::TL::TypeAt<TypeList,2>::Result& Get##T3() const { return Member<2>();}\
void Set##T3( const Loki::TL::TypeAt<TypeList,2>::Result& i_in) { Member<2>() = i_in;}

#define ACCESSORS_4(T1,T2,T3,T4) ACCESSORS_3(T1,T2,T3)\
const Loki::TL::TypeAt<TypeList,3>::Result& T4() const { return Member<3>();}\
Loki::TL::TypeAt<TypeList,3>::Result& T4() { return Member<3>();}\
const Loki::TL::TypeAt<TypeList,3>::Result& Get##T4() const { return Member<3>();}\
void Set##T4( const Loki::TL::TypeAt<TypeList,3>::Result& i_in) { Member<3>() = i_in;}

#define ACCESSORS_5(T1,T2,T3,T4,T5) ACCESSORS_4(T1,T2,T3,T4)\
const Loki::TL::TypeAt<TypeList,4>::Result& T5() const { return Member<4>();}\
Loki::TL::TypeAt<TypeList,4>::Result& T5() { return Member<4>();}\
const Loki::TL::TypeAt<TypeList,4>::Result& Get##T5() const { return Member<4>();}\
void Set##T5( const Loki::TL::TypeAt<TypeList,4>::Result& i_in) { Member<4>() = i_in;}

#define ACCESSORS_6(T1,T2,T3,T4,T5,T6) ACCESSORS_5(T1,T2,T3,T4,T5)\
const Loki::TL::TypeAt<TypeList,5>::Result& T6() const { return Member<5>();}\
Loki::TL::TypeAt<TypeList,5>::Result& T6() { return Member<5>();}\
const Loki::TL::TypeAt<TypeList,5>::Result& Get##T6() const { return Member<5>();}\
void Set##T6( const Loki::TL::TypeAt<TypeList,5>::Result& i_in) { Member<5>() = i_in;}

#define ACCESSORS_7(T1,T2,T3,T4,T5,T6,T7) ACCESSORS_6(T1,T2,T3,T4,T5,T6)\
const Loki::TL::TypeAt<TypeList,6>::Result& T7() const { return Member<6>();}\
Loki::TL::TypeAt<TypeList,6>::Result& T7() { return Member<6>();}\
const Loki::TL::TypeAt<TypeList,6>::Result& Get##T7() const { return Member<6>();}\
void Set##T7( const Loki::TL::TypeAt<TypeList,6>::Result& i_in) { Member<6>() = i_in;}

#define ACCESSORS_8(T1,T2,T3,T4,T5,T6,T7,T8) ACCESSORS_7(T1,T2,T3,T4,T5,T6,T7)\
const Loki::TL::TypeAt<TypeList,7>::Result& T8() const { return Member<7>();}\
Loki::TL::TypeAt<TypeList,7>::Result& T8() { return Member<7>();}\
const Loki::TL::TypeAt<TypeList,7>::Result& Get##T8() const { return Member<7>();}\
void Set##T8( const Loki::TL::TypeAt<TypeList,7>::Result& i_in) { Member<7>() = i_in;}

#define ACCESSORS_9(T1,T2,T3,T4,T5,T6,T7,T8,T9) ACCESSORS_8(T1,T2,T3,T4,T5,T6,T7,T8)\
const Loki::TL::TypeAt<TypeList,8>::Result& T9() const { return Member<8>();}\
Loki::TL::TypeAt<TypeList,8>::Result& T9() { return Member<8>();}\
const Loki::TL::TypeAt<TypeList,8>::Result& Get##T9() const { return Member<8>();}\
void Set##T9( const Loki::TL::TypeAt<TypeList,8>::Result& i_in) { Member<8>() = i_in;}

#define ACCESSORS_10(T1,T2,T3,T4,T5,T6,T7,T8,T9,T10) ACCESSORS_9(T1,T2,T3,T4,T5,T6,T7,T8,T9)\
const Loki::TL::TypeAt<TypeList,9>::Result& T10() const { return Member<9>();}\
Loki::TL::TypeAt<TypeList,9>::Result& T10() { return Member<9>();}\
const Loki::TL::TypeAt<TypeList,9>::Result& Get##T10() const { return Member<9>();}\
void Set##T10( const Loki::TL::TypeAt<TypeList,9>::Result& i_in) { Member<9>() = i_in;}

#define ACCESSORS_11(T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11) ACCESSORS_10(T1,T2,T3,T4,T5,T6,T7,T8,T9,T10)\
const Loki::TL::TypeAt<TypeList,10>::Result& T11() const { return Member<10>();}\
Loki::TL::TypeAt<TypeList,10>::Result& T11() { return Member<10>();}\
const Loki::TL::TypeAt<TypeList,10>::Result& Get##T11() const { return Member<10>();}\
void Set##T11( const Loki::TL::TypeAt<TypeList,10>::Result& i_in) { Member<10>() = i_in;}

#endif