#ifndef PSF_TESTBASE_HPP
#define PSF_TESTBASE_HPP

#include "Compound.hpp"
#include "Serializer.hpp"
#include "DeSerializer.hpp"

#include <list>
#include <vector>
#include <map>
#include <set>
#include <string>

typedef TYPELIST_3(int,bool,float) SimpleTypes;
class CSimple : public PSF::TCompound<SimpleTypes,CSimple>
{
public:
	ACCESSORS_3( Int, Bool, Float);
};

typedef TYPELIST_2(CSimple,std::list<CSimple>) ComplexTypes;
class CComplex : public PSF::TCompound<ComplexTypes,CComplex>
{
public:
	ACCESSORS_2( Simple, List);
};

typedef TYPELIST_3(int,std::wstring,float) BaseTypes;
class CBase : public PSF::TCompound<BaseTypes,CBase>
{
public:
	ACCESSORS_3( BaseMember1, BaseMember2, BaseMember3);
};

typedef TYPELIST_2(CSimple,int) SuperTypes;
class CSuper : public PSF::TCompound<SuperTypes,CSuper,CBase>
{
public:
	ACCESSORS_2( Member1, Member2);
};

class CTestBase
{
public:
	void DefaultInit();
	void DoSerialize( PSF::ISerializer& i_serializer);
	void DoDeSerialize( PSF::IDeSerializer& i_deserializer);
	void DoInverseDeSerialize( PSF::IDeSerializer& i_deserializer);

	// Various data types used for testing
	// POD like types
	std::wstring m_wstr;
	int m_nInt;
	unsigned int m_nUnsignedInt;
	float m_fFloat;
	bool m_bBool;

	// Simple class 
	CSimple m_simple;

	// Complex class 
	CComplex m_complex;

	// class that inherits from another compound class
	CSuper m_super;

	// standard containers
	std::list<int> m_list;
	std::vector<int> m_vector;
	std::map<int,int> m_map;
	std::set<int> m_set;
};


#endif